/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSPortNameServer;

import dstep.foundation.NSObject;
import dstep.foundation.NSPort;
import dstep.foundation.NSString;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

class NSPortNameServer : NSObject
{
	mixin (ObjcWrap);
	
	this ()
	{
		super(typeof(this).alloc.init.objcObject);
	}

	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}

	static NSPortNameServer systemDefaultPortNameServer ()
	{
		return invokeObjcSuperClass!(NSPortNameServer, "systemDefaultPortNameServer");
	}

	NSPort portForName (NSString name)
	{
		return invokeObjcSelf!(NSPort, "portForName:", NSString)(name);
	}

	NSPort portForName (NSString name, NSString host)
	{
		return invokeObjcSelf!(NSPort, "portForName:host:", NSString, NSString)(name, host);
	}

	bool registerPort (NSPort port, NSString name)
	{
		return invokeObjcSelf!(bool, "registerPort:name:", NSPort, NSString)(port, name);
	}

	bool removePortForName (NSString name)
	{
		return invokeObjcSelf!(bool, "removePortForName:", NSString)(name);
	}
}

class NSMachBootstrapServer : NSPortNameServer
{
	mixin (ObjcWrap);
	
	this ()
	{
		super(typeof(this).alloc.init.objcObject);
	}

	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}

	static Object sharedInstance ()
	{
		return invokeObjcSuperClass!(Object, "sharedInstance");
	}

	NSPort portForName (NSString name)
	{
		return invokeObjcSelf!(NSPort, "portForName:", NSString)(name);
	}

	NSPort portForName (NSString name, NSString host)
	{
		return invokeObjcSelf!(NSPort, "portForName:host:", NSString, NSString)(name, host);
	}

	bool registerPort (NSPort port, NSString name)
	{
		return invokeObjcSelf!(bool, "registerPort:name:", NSPort, NSString)(port, name);
	}

	NSPort servicePortWithName (NSString name)
	{
		return invokeObjcSelf!(NSPort, "servicePortWithName:", NSString)(name);
	}
}

class NSSocketPortNameServer : NSPortNameServer
{
	mixin (ObjcWrap);
	
	this ()
	{
		super(typeof(this).alloc.init.objcObject);
	}
	
	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}

	static Object sharedInstance ()
	{
		return invokeObjcSuperClass!(Object, "sharedInstance");
	}

	NSPort portForName (NSString name)
	{
		return invokeObjcSelf!(NSPort, "portForName:", NSString)(name);
	}

	NSPort portForName (NSString name, NSString host)
	{
		return invokeObjcSelf!(NSPort, "portForName:host:", NSString, NSString)(name, host);
	}

	bool registerPort (NSPort port, NSString name)
	{
		return invokeObjcSelf!(bool, "registerPort:name:", NSPort, NSString)(port, name);
	}

	bool removePortForName (NSString name)
	{
		return invokeObjcSelf!(bool, "removePortForName:", NSString)(name);
	}

	NSPort portForName (NSString name, NSString host, ushort portNumber)
	{
		return invokeObjcSelf!(NSPort, "portForName:host:nameServerPortNumber:", NSString, NSString, ushort)(name, host, portNumber);
	}

	bool registerPort (NSPort port, NSString name, ushort portNumber)
	{
		return invokeObjcSelf!(bool, "registerPort:name:nameServerPortNumber:", NSPort, NSString, ushort)(port, name, portNumber);
	}

	void setDefaultNameServerPortNumber (ushort portNumber)
	{
		return invokeObjcSelf!(void, "setDefaultNameServerPortNumber:", ushort)(portNumber);
	}

	ushort defaultNameServerPortNumber ()
	{
		return invokeObjcSelf!(ushort, "defaultNameServerPortNumber");
	}
}

class NSMessagePortNameServer : NSPortNameServer
{
	mixin (ObjcWrap);
	
	this ()
	{
		super(typeof(this).alloc.init.objcObject);
	}
	
	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}

	static Object sharedInstance ()
	{
		return invokeObjcSuperClass!(Object, "sharedInstance");
	}

	NSPort portForName (NSString name)
	{
		return invokeObjcSelf!(NSPort, "portForName:", NSString)(name);
	}

	NSPort portForName (NSString name, NSString host)
	{
		return invokeObjcSelf!(NSPort, "portForName:host:", NSString, NSString)(name, host);
	}
}

