/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSRunLoop;

import dstep.corefoundation.CFRunLoop;
import dstep.foundation.NSArray;
import dstep.foundation.NSDate;
import dstep.foundation.NSObjCRuntime;
import dstep.foundation.NSObject;
import dstep.foundation.NSPort;
import dstep.foundation.NSString;
import dstep.foundation.NSTimer;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

import bindings = dstep.foundation.NSRunLoop_bindings;

private
{
	NSString NSDefaultRunLoopMode_;
	NSString NSRunLoopCommonModes_;
}

NSString NSDefaultRunLoopMode ()
{
	if (NSDefaultRunLoopMode_)
		return NSDefaultRunLoopMode_;
		
	return NSDefaultRunLoopMode_ = new NSString(bindings.NSDefaultRunLoopMode);
}

NSString NSRunLoopCommonModes ()
{
	if (NSRunLoopCommonModes_)
		return NSRunLoopCommonModes_;
		
	return NSRunLoopCommonModes_ = new NSString(bindings.NSRunLoopCommonModes);
}

const TNSOrderedPerform = `

	void performSelector (SEL aSelector, Object target, Object arg, NSUInteger order, NSArray modes)
	{
		return invokeObjcSelf!(void, "performSelector:target:argument:order:modes:", SEL, Object, Object, NSUInteger, NSArray)(aSelector, target, arg, order, modes);
	}

	void cancelPerformSelector (SEL aSelector, Object target, Object arg)
	{
		return invokeObjcSelf!(void, "cancelPerformSelector:target:argument:", SEL, Object, Object)(aSelector, target, arg);
	}

	void cancelPerformSelectorsWithTarget (Object target)
	{
		return invokeObjcSelf!(void, "cancelPerformSelectorsWithTarget:", Object)(target);
	}
`;

const TNSDelayedPerforming = `

	void performSelector (SEL aSelector, Object anArgument, double delay, NSArray modes)
	{
		return invokeObjcSelf!(void, "performSelector:withObject:afterDelay:inModes:", SEL, Object, double, NSArray)(aSelector, anArgument, delay, modes);
	}
	
	void performSelector (SEL aSelector, Object anArgument, double delay)
	{
		return invokeObjcSelf!(void, "performSelector:withObject:afterDelay:", SEL, Object, double)(aSelector, anArgument, delay);
	}
	
	static void cancelPreviousPerformRequestsWithTarget (Object aTarget, SEL aSelector, Object anArgument)
	{
		return invokeObjcSelfClass!(void, "cancelPreviousPerformRequestsWithTarget:selector:object:", Object, SEL, Object)(aTarget, aSelector, anArgument);
	}
	
	static void cancelPreviousPerformRequestsWithTarget (Object aTarget)
	{
		return invokeObjcSelfClass!(void, "cancelPreviousPerformRequestsWithTarget:", Object)(aTarget);
	}
	
	//mixin ObjcBindMethod!(performSelector, void, "performSelector:withObject:afterDelay:inModes:", SEL, Object, double, NSArray);
	//mixin ObjcBindMethod!(performSelector, void, "performSelector:withObject:afterDelay:", SEL, Object, double);
	//mixin ObjcBindClassMethod!(cancelPreviousPerformRequestsWithTarget, "cancelPreviousPerformRequestsWithTarget:selector:object:");
	//mixin ObjcBindClassMethod!(cancelPreviousPerformRequestsWithTarget, "cancelPreviousPerformRequestsWithTarget:");
`;

const TNSRunLoopConveniences = `

	void run ()
	{
		return invokeObjcSelf!(void, "run");
	}

	void runUntilDate (NSDate limitDate)
	{
		return invokeObjcSelf!(void, "runUntilDate:", NSDate)(limitDate);
	}

	bool runMode (NSString mode, NSDate limitDate)
	{
		return invokeObjcSelf!(bool, "runMode:beforeDate:", NSString, NSDate)(mode, limitDate);
	}

	void configureAsServer ()
	{
		return invokeObjcSelf!(void, "configureAsServer");
	}
`;

class NSRunLoop : NSObject
{
	mixin (ObjcWrap);
	
	this ()
	{
		super(typeof(this).alloc.init.objcObject);
	}
	
	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}

	static NSRunLoop currentRunLoop ()
	{
		return invokeObjcSuperClass!(NSRunLoop, "currentRunLoop");
	}

	static NSRunLoop mainRunLoop ()
	{
		return invokeObjcSuperClass!(NSRunLoop, "mainRunLoop");
	}

	NSString currentMode ()
	{
		return invokeObjcSelf!(NSString, "currentMode");
	}

	CFRunLoopRef getCFRunLoop ()
	{
		return invokeObjcSelf!(CFRunLoopRef, "getCFRunLoop");
	}

	void addTimer (NSTimer timer, NSString mode)
	{
		return invokeObjcSelf!(void, "addTimer:forMode:", NSTimer, NSString)(timer, mode);
	}

	void addPort (NSPort aPort, NSString mode)
	{
		return invokeObjcSelf!(void, "addPort:forMode:", NSPort, NSString)(aPort, mode);
	}

	void removePort (NSPort aPort, NSString mode)
	{
		return invokeObjcSelf!(void, "removePort:forMode:", NSPort, NSString)(aPort, mode);
	}

	NSDate limitDateForMode (NSString mode)
	{
		return invokeObjcSelf!(NSDate, "limitDateForMode:", NSString)(mode);
	}

	void acceptInputForMode (NSString mode, NSDate limitDate)
	{
		return invokeObjcSelf!(void, "acceptInputForMode:beforeDate:", NSString, NSDate)(mode, limitDate);
	}
	
	// TNSOrderedPerform
	void performSelector (SEL aSelector, Object target, Object arg, NSUInteger order, NSArray modes)
	{
		return invokeObjcSelf!(void, "performSelector:target:argument:order:modes:", SEL, Object, Object, NSUInteger, NSArray)(aSelector, target, arg, order, modes);
	}
	
	void cancelPerformSelector (SEL aSelector, Object target, Object arg)
	{
		return invokeObjcSelf!(void, "cancelPerformSelector:target:argument:", SEL, Object, Object)(aSelector, target, arg);
	}
	
	void cancelPerformSelectorsWithTarget (Object target)
	{
		return invokeObjcSelf!(void, "cancelPerformSelectorsWithTarget:", Object)(target);
	}
	
	// TNSRunLoopConveniences
	void run ()
	{
		return invokeObjcSelf!(void, "run");
	}
	
	void runUntilDate (NSDate limitDate)
	{
		return invokeObjcSelf!(void, "runUntilDate:", NSDate)(limitDate);
	}
	
	bool runMode (NSString mode, NSDate limitDate)
	{
		return invokeObjcSelf!(bool, "runMode:beforeDate:", NSString, NSDate)(mode, limitDate);
	}
	
	void configureAsServer ()
	{
		return invokeObjcSelf!(void, "configureAsServer");
	}
}